@@ -90,6 +90,7 @@
         'bedrock': buildTypeDefinition('icon-bedrock', 'AWS Bedrock'),
         'mosaicml': buildTypeDefinition('icon-mosaicml', 'MosaicML (deprecated)'),
         'stabilityai': buildTypeDefinition('dku-icon-stability-ai', 'Stability AI', undefined, true),
+        'nvidia-nim': buildTypeDefinition('dku-icon-nvidia', 'NVIDIA NIM'), // key needs to be lowercase, see typeProcessor in services.js
         'sagemaker-genericllm': buildTypeDefinition('icon-model-sagemaker', 'Amazon SageMaker LLM'),
         'vertexaillm': buildTypeDefinition('icon-vertex', 'Vertex Generative AI'),
         'databricksllm': buildTypeDefinition('icon-dku-databricks', 'Databricks Mosaic AI'),
@@ -438,7 +439,7 @@
         'icon-visual_prep_list_folder_contents_recipe': 'dku-icon-recipe-list-folder-contents-circle-fill',
         'icon-visual_prep_list_access_recipe': 'dku-icon-recipe-list-access',
         'icon-visual_prep_pivot_recipe': 'dku-icon-recipe-pivot-circle-fill',
-        'icon-circle': 'dku-icon-circle-fill',
+        'icon-circle': 'dku-icon-recipe-type',
         'icon-visual_prep_filter-sample_recipe': 'dku-icon-recipe-filter-circle-fill',
         'icon-visual_prep_split_recipe': 'dku-icon-recipe-split-circle-fill',
         'icon-visual_prep_sync_recipe': 'dku-icon-recipe-sync-right',
