@@ -333,8 +333,8 @@
             for (let i = 0; i < $scope.dashboards.length; i++) {
                 if ($scope.dashboards[i].id == $scope.dashboard.id) {
                     $scope.pinningOrder = {
-                        dashboard: $scope.dashboards[i],
-                        page: $scope.dashboards[i].pages[$scope.uiState.currentPageIdx],
+                        dashboard: $scope.dashboard,
+                        page: $scope.dashboard.pages[$scope.uiState.currentPageIdx],
                         useInsightTheme: false
                     };
                     $scope.pinningOrders = [$scope.pinningOrder];
@@ -365,10 +365,10 @@
          * Methods used if copying/moving the tile in the current dashboard
          */
 
-        function moveCopyTile(destinationPageId) {
+        function moveCopyTile(destinationPageCid) {
             let destinationPage = null;
             for (let i=0; i<$scope.dashboard.pages.length; i++) {
-                if ($scope.dashboard.pages[i].id == destinationPageId || $scope.dashboard.pages[i].$$hashKey == destinationPageId) {
+                if ($scope.dashboard.pages[i].cid == destinationPageCid || $scope.dashboard.pages[i].$$hashKey == destinationPageCid) {
                     destinationPage = $scope.dashboard.pages[i];
                     break;
                 }
@@ -428,7 +428,7 @@
 
         $scope.validate = function() {
             if ($scope.pinningOrder.dashboard.id == $scope.dashboard.id) {
-                moveCopyTile($scope.pinningOrder.page.id || $scope.pinningOrder.page.$$hashKey);
+                moveCopyTile($scope.pinningOrder.page.cid || $scope.pinningOrder.page.$$hashKey);
             } else {
                 $scope.sendPinningOrders();
             }
@@ -447,7 +447,7 @@
 
             if (!$scope.keepOriginal) {
                 return $scope.pinningOrder.dashboard.pages.filter(function(page) {
-                    return $scope.page.id != page.id;
+                    return $scope.page.cid != page.cid;
                 });
             } else {
                 return $scope.pinningOrder.dashboard.pages;
@@ -539,12 +539,18 @@
                 .success(function(data) {
                     $scope.dashboards = data.dashboards;
                     $scope.allDashboardsCount = data.allDashboardsCount;
-                    //listing where insight got already pinned
-                    if (insight && insight.id) {
-                        $scope.dashboards.forEach(function(dashboard, index) {
+
+                    // Use local state of current dashboard to take into account slides being added / removed
+                    $scope.dashboards.forEach(function(dashboard, index) {
+                        if ($scope.dashboard && $scope.dashboard.id === dashboard.id) {
+                            $scope.dashboards[index] = $scope.dashboard;
+                        }
+
+                        //listing where insight got already pinned
+                        if (insight && insight.id) {
                             dashboard.pages.forEach(function(page, pageIndex) {
                                 page.index = pageIndex;
-                                page.grid.tiles.forEach(function(tile) {
+                                TileUtils.getFlattenTileList(page.grid.tiles).forEach((tile) => {
                                     if (tile.insightId == insight.id) {
                                         $scope.existingPinningList.push({
                                             'dashboard': dashboard,
@@ -553,12 +559,9 @@
                                     }
                                 });
                             });
-                            // Use local state of current dashboard to take into account slides being added / removed
-                            if ($scope.dashboard && $scope.dashboard.id === dashboard.id) {
-                                $scope.dashboards[index] = $scope.dashboard;
-                            }
-                        });
-                    }
+                        }
+                    });
+
                     if ($scope.dashboards.length > 0) {
                         $scope.addPinningOrder();
                     }
@@ -607,7 +610,7 @@
             $scope.pinningOrders.push({
                 dashboard: projectHasDashboards ? $scope.dashboards[0] : CreateAndPinInsight.getDashboardTemplate(),
                 page: projectHasDashboards ? $scope.dashboards[0].pages[0] : CreateAndPinInsight.getPageTemplate(),
-                useInsightTheme: !$scope.dashboards[0].theme
+                useInsightTheme: projectHasDashboards ? !$scope.dashboards[0].theme : true
             });
         };
 
