@@ -699,6 +699,21 @@ app.factory("Dialogs", ["CreateModalFromTemplate", "$q", "$state", "$timeout","D
             });
             return deferred.promise;
         },
+        errorUnsafeHTML : function($scope, title, text) {
+            var deferred = $q.defer();
+            CreateModalFromTemplate("/templates/dialogs/error-dialog-unsafe-html.html", $scope, "ConfirmDialogController", function(newScope) {
+                newScope.acceptDeferred = deferred;
+                newScope.title = title;
+                newScope.text = text;
+                newScope.$on("$destroy",function() {
+                    if(newScope.acceptDeferred) {
+                        newScope.acceptDeferred.reject();
+                    }
+                    newScope.acceptDeferred = null;
+                });
+            });
+            return deferred.promise;
+        },
         confirm : function($scope, title, text, options) {
             var deferred = $q.defer();
             CreateModalFromTemplate("/templates/dialogs/confirm-dialog.html", $scope, "ConfirmDialogController", function(newScope) {
