@@ -3,6 +3,21 @@
 
     var app = angular.module('dataiku.directives.snippetEditor', ['dataiku.filters', 'dataiku.services', 'ui.keypress', 'dataiku.common.lists']);
 
+	app.directive('libDocumentationSwitch', function() {
+		return {
+			scope: {
+				editorParams: '='
+			},
+			controller : function($scope) {
+				//showing and hiding the codeSnippetEditorSampleSelector
+				$scope.switchLibsDocumentationVisibility = function() {
+					$scope.editorParams.libsDocumentationVisible = !$scope.editorParams.libsDocumentationVisible;
+				}
+			},
+			templateUrl : '/templates/widgets/lib-documentation-switch.html'
+		}
+	});
+
 	app.directive('codeSnippetEditorSwitch', function() {
 		return {
 			scope: {
@@ -100,7 +115,7 @@
 	            resolveCodeForInsertionFunc : '=?',
 	            displayed : '='
 	        },
-	        controller : function($scope, $element, $timeout, DataikuAPI, ListFilter, WT1, CodeMirrorSettingService) {
+	        controller : function($scope, $element, $timeout, $stateParams, DataikuAPI, ListFilter, WT1, CodeMirrorSettingService) {
 	        	//Checking editorOptions parameters
 	        	if (typeof($scope.editorOptions)==='undefined') {
                     $scope.editorOptions = CodeMirrorSettingService.get('text/x-python', {onLoad: function(cm) {$scope.codeMirror = cm;}});
@@ -117,6 +132,7 @@
 	        	}
 
 	        	//Preparing parameters for nested directive
+	        	$scope.projectKey = $stateParams.projectKey;
 
 	        	//--insertCodeFunc: method used to insert code sample into code mirror instance
 	        	var insertCode = function (codeToInsert) {
@@ -173,6 +189,10 @@
 	        	$scope.snippetTags = [];
 	    		$scope.snippetTagsMap = {};
 
+				//--autogenerated doc
+				$scope.autogeneratedDocumentationEnabled = ($scope.sampleType === 'python' && $rootScope.appConfig.autogeneratedDocumentationEnabled && $rootScope.featureFlagEnabled('docportal'));
+				$scope.libsDocumentationVisible = false;
+
 	        	//Wrapping parameters needed by the view for scope issue
 	        	$scope.editorParams = $scope;
 
@@ -188,6 +208,23 @@
 	        	        }
 	        		});
 	        	}, true);
+
+				$scope.startResize = function (event) {
+					const resizableDiv = document.getElementById("resizableDiv");
+					event.preventDefault();
+
+					function resize(event) {
+						resizableDiv.style.width = (resizableDiv.offsetWidth - event.movementX) + "px";
+					}
+
+					function stopResize() {
+						document.removeEventListener("mousemove", resize);
+						document.removeEventListener("mouseup", stopResize);
+					}
+
+					document.addEventListener("mousemove", resize);
+					document.addEventListener("mouseup", stopResize);
+				}
 	        }
 		}
 	});
