@@ -3,7 +3,7 @@
 
 const app = angular.module('dataiku.promptstudios', ['dataiku.services', 'dataiku.filters']);
 
-app.factory("PromptUtils", function(Logger, CreateModalFromTemplate, $filter, $dkuSanitize, COLOR_PALETTES, $stateParams, DataikuAPI, WT1, SmartId) {
+app.factory("PromptUtils", function(Logger, CreateModalFromTemplate, $filter, $dkuSanitize, COLOR_PALETTES, $stateParams, DataikuAPI, WT1, SmartId, RALLM_SEARCH_INPUT_STRATEGY_MAP) {
     const svc = {};
 
     svc.getTemperatureRange = function(llm) {
@@ -385,9 +385,8 @@ app.factory("PromptUtils", function(Logger, CreateModalFromTemplate, $filter, $d
             return messages;
         }
 
-        const llmIdParts = llmId.split(":");
-        const isRAG = llmIdParts[0] === "retrieval-augmented-llm";
-        if (isRAG) {
+        if (svc.isRAG(llmId)) {
+            const llmIdParts = llmId.split(":");
             DataikuAPI.savedmodels.getFullInfo($stateParams.projectKey, SmartId.create(llmIdParts[1], $stateParams.projectKey)).success(function(res) {
                 const ragActiveVersion = res.model.inlineVersions.find(inlineVersion => inlineVersion.versionId === res.model.activeVersion);
                 CreateModalFromTemplate("/templates/promptstudios/prompt-studio-see-raw-prompt.html", scope, null, function(newScope) {
@@ -601,6 +600,20 @@ app.factory("PromptUtils", function(Logger, CreateModalFromTemplate, $filter, $d
         return changes;
     };
 
+    svc.isRAG = function(llmId) {
+        return llmId.startsWith("retrieval-augmented-llm:");
+    }
+
+    svc.setWarningWhenUsedInChat = function(llmId, callback) {
+        if (llmId && svc.isRAG(llmId)) {
+            const llmIdParts = llmId.split(":");
+            DataikuAPI.savedmodels.getFullInfo($stateParams.projectKey, SmartId.create(llmIdParts[1], $stateParams.projectKey)).success(function (res) {
+                const inline_version = res.model.inlineVersions.find(inlineVersion => inlineVersion.versionId === res.model.activeVersion);
+                callback(RALLM_SEARCH_INPUT_STRATEGY_MAP[inline_version.ragllmSettings.searchInputStrategySettings.strategy].warnIfUsedInChat);
+            });
+        }
+    }
+
     return svc;
 });
 
@@ -807,6 +820,9 @@ app.controller('PromptStudioController', function ($scope, $state, $stateParams,
                 $scope.getPromptDatasetPreview();
             }
         }).error(setErrorInScope.bind($scope));
+
+        PromptUtils.setWarningWhenUsedInChat($scope.activePromptStudioPrompt.llmId, function(warn) {$scope.warnAboutRaLlmSearchMode=warn})
+
     };
 
     $scope.promptStats = {};
@@ -908,6 +924,7 @@ app.controller('PromptStudioController', function ($scope, $state, $stateParams,
                     name: PromptUtils.getLLMIcon(($scope.activeLLM || {}).type),
                     style: PromptUtils.getLLMColorStyle($scope.activePromptStudioPrompt.llmId, $scope.availableLLMs)
                 };
+                PromptUtils.setWarningWhenUsedInChat($scope.activePromptStudioPrompt.llmId, function(warn) {$scope.warnAboutRaLlmSearchMode=warn});
             });
 
             WT1.event("prompt-studio-llms-load", {
@@ -1685,7 +1702,7 @@ app.component('promptStudioResponseActions', {
         getLlmOutput: '&'
     },
     templateUrl: '/templates/promptstudios/prompt-studio-response-actions.html',
-    controller: function(PromptUtils, ClipboardUtils) {
+    controller: function(PromptUtils, ClipboardUtils, TraceExplorerService) {
         const $ctrl = this;
 
         $ctrl.canCreateExampleFromResponse = function() {
@@ -1711,6 +1728,10 @@ app.component('promptStudioResponseActions', {
         $ctrl.copyResponseToClipboard = function() {
             ClipboardUtils.copyToClipboard($ctrl.getLlmOutput($ctrl.singleResponse), "Response copied to clipboard");
         };
+        
+        $ctrl.openTraceExplorer = function() {
+            TraceExplorerService.openTraceExplorer($ctrl.singleResponse.fullTrace)
+        };
     }
 });
 
@@ -1778,7 +1799,7 @@ app.component('promptLlmCompletionSettings', {
             }
 
             if (changes.completionSettings) {
-                $ctrl.jsonMode = $ctrl.completionSettings.responseFormat && $ctrl.completionSettings.responseFormat.type === 'json';
+                $ctrl.jsonMode = $ctrl.completionSettings?.responseFormat?.type === 'json';
             }
         };
 
@@ -2063,7 +2084,7 @@ app.component('promptImages', {
         $ctrl.handleImageLoad = () => {
             loadedImageCount++;
             if (loadedImageCount >= $ctrl.displayedPaths.length) {
-                $ctrl.onRendered && $ctrl.onRendered();
+                $ctrl.onRendered?.();
             }
         };
     }
@@ -2088,7 +2109,7 @@ app.component('promptImage', {
         };
 
         $ctrl.onLoad = function() {
-            $ctrl.onImageLoad && $ctrl.onImageLoad();
+            $ctrl.onImageLoad?.();
         };
 
         $ctrl.$onChanges = function(changes) {
