@@ -120,7 +120,6 @@
             .then(() => {
                 buildInitialColumnsMatchForInput(inputDesc);
                 $scope.params.virtualInputs.push(inputDesc);
-                updateUnionSchema();
                 $scope.updateColumnsSelection();
                 $scope.updateRecipeStatusLater();
             });
@@ -343,6 +342,11 @@
         };
 
         $scope.updateColumnsSelection = function() {
+            updateUnionSchema();
+            if(!$scope.params) {
+                return;
+            }
+
             // clear the columnsMatch value in each input if the mode is not REMAP
             if ($scope.params.mode != 'REMAP') {
                 $scope.params.virtualInputs.forEach(vi => { delete vi.columnsMatch; });
@@ -498,7 +502,6 @@
             });
 
 
-            updateUnionSchema();
             $scope.updateColumnsSelection();
             if ($scope.params.mode == 'FROM_INDEX') {
                 $scope.syncSelectedColumns();
@@ -522,20 +525,12 @@
             var columns = [];
             var columnNames = [];
             $scope.unionSchema = $scope.unionSchema || [];
-            var previouslySelected = $scope.unionSchema.filter(function(col){
-                return col.selected;
-            }).map(function(col) {
-                return col.name;
-            });
             RecipesUtils.getFlatInputsList($scope.recipe).forEach(function(input) {
                 $scope.flatInputRefs.push(input.ref);
                 $scope.getColumns(input.ref).forEach(function(column) {
                     if (columnNames.indexOf(column.name) < 0) {
                         columns.push(column);
                         columnNames.push(column.name);
-                        if (previouslySelected[column.name]) {
-                            column.selected = true;
-                        }
                     }
                 });
             });
@@ -553,9 +548,7 @@
                 if ($scope.script.data) {
                     $scope.params = JSON.parse($scope.script.data);
 
-                    updateUnionSchema();
                     $scope.updateColumnsSelection();
-                    updateSelectedColumns();
 
                     visualCtrl.saveServerParams(); //keep for dirtyness detection
 
@@ -571,12 +564,12 @@
             $scope.$watch("script.data", onScriptChanged);
             $scope.$watch("recipe.inputs", function() {
                 DatasetUtils.updateRecipeComputables($scope, $scope.recipe, $stateParams.projectKey, $stateParams.projectKey)
-                    .then(_ => updateUnionSchema());
+                    .then(_ => () => $scope.updateColumnsSelection());
             }, true);
             $scope.$watch("params.postFilter", $scope.updateRecipeStatusLater, true);
             $scope.$watch("params.virtualInputs", $scope.updateRecipeStatusLater, true);
             $scope.$watch("params.selectedColumns", $scope.updateRecipeStatusLater, true);
-            $scope.$watch("params.mode", updateUnionSchema, true);
+            $scope.$watch("params.mode", () => $scope.updateColumnsSelection(), true);
             // don't pass $scope.updateRecipeStatus as the callback, because it will get parameters which are absolutely not what is expected:
             $scope.$watch("unionSchema", function(){$scope.updateRecipeStatusLater()}, true); //call updateRecipeStatus without args!
             $scope.$watch("recipe.outputs", function(){
@@ -587,7 +580,7 @@
                 $scope.updateRecipeStatusLater();
             }, true);
 
-            updateUnionSchema();
+            $scope.updateColumnsSelection();
             onScriptChanged($scope.script.data);
         };
 
