@@ -9,7 +9,7 @@
      * Service responsible for computing pivot requests by chart type compatible with the get-pivot-response api.
      * (!) This file previously was in static/dataiku/js/simple_report/chart_compute_requests.js
      */
-    app.service('ChartRequestComputer', function(ChartDimension, LoggerProvider, BinnedXYUtils, _GeometryCommon, ChartColorUtils, ChartStoreFactory, CHART_TYPES, CHART_VARIANTS, ChartUADimension, MultiplotUtils, ChartUsableColumns, ChartCustomMeasures, ReferenceLines, ColumnAvailability, ChartFeatures, ColorMode) {
+    app.service('ChartRequestComputer', function(ChartDimension, LoggerProvider, BinnedXYUtils, _GeometryCommon, ChartColorUtils, ChartStoreFactory, CHART_TYPES, CHART_VARIANTS, ChartUADimension, MultiplotUtils, ChartUsableColumns, ChartCustomMeasures, ColumnAvailability, ChartFeatures, ColorMode) {
         const Logger = LoggerProvider.getLogger('chartrequest');
 
         /* Helpers */
@@ -18,7 +18,7 @@
             return array && array.length > 0;
         };
 
-        const makeAggregatedAxis = function(dim, chartDef, isInteractiveDateDimension = false, isMainInteractiveDateAxis = false) {
+        const makeAggregatedAxis = function(dim, chartDef, isMainInteractiveDateAxis = false) {
             dim = angular.copy(dim);
             const axis = {
                 column: dim.column,
@@ -773,6 +773,10 @@
                         return computeStdAggregated(modifiedChartDef, chartSpecific, computeSubTotals);
                     }
                     case CHART_TYPES.PIVOT_TABLE:
+                        if(!has(chartDef.xDimension) && !has(chartDef.yDimension)) {
+                            return computeKpi(chartDef);
+                        }
+                        return computeHierarchical(chartDef);
                     case CHART_TYPES.SANKEY:
                     case CHART_TYPES.TREEMAP:
                         return computeHierarchical(chartDef);
