@@ -518,8 +518,8 @@
                 manageLink: "=",
                 noTagIcon: '<',
                 editable: '=?',
-                responsive: '=?'
-
+                responsive: '=?',
+                tagsTooltip: '<'
             },
             link: function(scope, element, attrs) {
                 scope.translate = translate;
@@ -626,6 +626,12 @@
                     $timeout(setInputFocus, 100);
                 }
 
+                scope.getTagTooltip = function(tagName) {
+                    if (!scope.tagsTooltip) { return ""; }
+                    if (!scope.tagsTooltip[tagName]) { return ""; }
+                    return scope.tagsTooltip[tagName];
+                }
+
                 scope.saveEdit = function(e) {
                     $("html").off("click.tagEdit");
                     eatEvent(e);
