@@ -17,7 +17,7 @@ def get_dataiku_user_uid():
             return "500"
         with open(path, "r") as f:
             for line in f.readlines():
-                match = re.match("^dku.container.dataiku.uid\s*=\s*(\w+)$", line)
+                match = re.match(r"^dku.container.dataiku.uid\s*=\s*(\w+)$", line)
                 if match:
                     return match.group(1)
     except Exception as e:
