@@ -11,13 +11,14 @@ from dataiku.doctor.diagnostics import diagnostics
 from dataiku.doctor.prediction.custom_scoring import check_customscore
 from dataiku.doctor.prediction.custom_scoring import get_custom_metric_scorefunc
 from dataiku.doctor.timeseries.preparation.resampling.utils import get_frequency
-from dataiku.doctor.timeseries.utils import JSONForecast
+from dataiku.doctor.timeseries.utils import JSONForecast, format_forecast_dates
 from dataiku.doctor.timeseries.utils import ModelForecast
 from dataiku.doctor.timeseries.utils import SINGLE_TIMESERIES_IDENTIFIER
 from dataiku.doctor.timeseries.utils import timeseries_iterator
 from dataiku.doctor.timeseries.utils import prefix_custom_metric_name
 from dataiku.doctor.timeseries.utils import get_dataframe_of_timeseries_identifier
 from dataiku.doctor.timeseries.utils import pretty_timeseries_identifiers
+from dataiku.doctor.timeseries.utils import build_quantile_column_name
 from dataiku.doctor.utils import dku_nonaninf
 from dataiku.doctor.utils import dku_average_nonan
 from dataiku.doctor.utils import dku_std_nonan
@@ -43,6 +44,13 @@ AGGREGATED_TIMESERIES_METRICS = SINGLE_TIMESERIES_METRICS + [
     "meanAbsoluteQuantileLoss",
     "meanWeightedQuantileLoss",
 ]
+SINGLE_TIMESERIES_METRICS_SKIP_SCORING = ["mse", "mape", "smape", "mae"]
+AGGREGATED_TIMESERIES_METRICS_SKIP_SCORING = SINGLE_TIMESERIES_METRICS_SKIP_SCORING + [
+    "rmse",
+    "nd",
+    "meanAbsoluteQuantileLoss",
+    "meanWeightedQuantileLoss",
+]
 
 PER_TIMESERIES_METRICS = "perTimeseriesMetrics"
 TIMESERIES_AGGREGATED_METRICS = "aggregatedMetrics"
@@ -51,11 +59,12 @@ TIMESERIES_AGGREGATED_METRICS = "aggregatedMetrics"
 class TimeseriesModelIntrinsicScorer:
     # TODO: inherit from PredictionModelIntrinsicScorer once refactored ?
 
-    def __init__(self, modeling_params, clf, algorithm, prediction_length, frequency, initial_intrinsic_perf_data):
+    def __init__(self, modeling_params, preprocessing_params, clf, algorithm, prediction_length, frequency, initial_intrinsic_perf_data):
         self.algorithm = algorithm
         self.clf = clf
 
         actual_params = algorithm.get_actual_params(modeling_params, clf, fit_params=None)
+        self.preprocessing_params = preprocessing_params
         self.resolved_actual_params = actual_params["resolved"]
 
         self.prediction_length = prediction_length
@@ -65,8 +74,10 @@ class TimeseriesModelIntrinsicScorer:
     def score(self):
         intrinsic_perf = self.initial_intrinsic_perf_data
 
-        intrinsic_perf["minTimeseriesSizeForScoring"] = self.algorithm.get_min_size_for_scoring(self.resolved_actual_params, self.prediction_length)
-        intrinsic_perf["maxUsedTimestepsForScoring"] = self.algorithm.get_max_used_timesteps_for_scoring(self.resolved_actual_params, self.prediction_length, self.frequency)
+        intrinsic_perf["minTimeseriesSizeForScoring"] = self.algorithm.get_min_size_for_scoring(
+            self.resolved_actual_params, self.preprocessing_params, self.prediction_length)
+        intrinsic_perf["maxUsedTimestepsForScoring"] = self.algorithm.get_max_used_timesteps_for_scoring(
+            self.resolved_actual_params, self.preprocessing_params, self.prediction_length, self.frequency)
 
         if self.algorithm.SUPPORTS_MODEL_COEFFICIENTS:
             coefficients_map, fixed_coefficients, variable_coefficients, external_features_coefficients = self.clf.get_coefficients_map_and_names()
@@ -139,7 +150,8 @@ class TimeseriesModelScorer(object):
         use_external_features,
         frequency,
         custom_metrics,
-        max_nb_forecast_timesteps=None
+        max_nb_forecast_timesteps=None,
+        skip_scoring=False #If True, the forecast will not be computed, and metrics will be calculated based on the already existing forecast
     ):
         self.target_variable = target_variable
         self.time_variable = time_variable
@@ -162,6 +174,7 @@ class TimeseriesModelScorer(object):
 
         self.custom_metrics = custom_metrics  # list of {"name": XXX, "metricCode": XXX, "greaterIsBetter", ...}
         self.max_nb_forecast_timesteps = max_nb_forecast_timesteps
+        self.skip_scoring = skip_scoring
 
     @classmethod
     def build(cls, core_params, metrics_params, use_external_features):
@@ -252,7 +265,7 @@ class TimeseriesModelScorer(object):
             if extra_steps_to_forecast > 0:
                 # this should only occur when using a custom train/test split
 
-                # past_df_of_timeseries_identifier now contains everthing but the last extra_steps_to_forecast rows
+                # past_df_of_timeseries_identifier now contains everything but the last extra_steps_to_forecast rows
                 # future_df_of_timeseries_identifier is just these rows
                 past_df_of_timeseries_identifier = pd.concat([
                     timeseries_identifier_historical_df,
@@ -301,12 +314,12 @@ class TimeseriesModelScorer(object):
         nb_timesteps_to_keep = self.max_nb_forecast_timesteps or len(forecast_values)
 
         # Slice y_true to match shortened timeseries_forecasts
-        y_true = test_df[self.target_variable][-nb_timesteps_to_keep:].to_numpy()
+        y_true = test_df[self.target_variable][-nb_timesteps_to_keep:].values
 
         naive_error = self._compute_single_timeseries_naive_error(historical_df, y_true)
 
         if append_forecasts:
-            max_ground_truth_date = pd.Timestamp(self.forecasts[timeseries_identifier][JSONForecast.GROUND_TRUTH_TIME].max())
+            max_ground_truth_date = pd.to_datetime(self.forecasts[timeseries_identifier][JSONForecast.GROUND_TRUTH_TIME]).max()
             if historical_df[self.time_variable].max() > max_ground_truth_date:
                 # the current train set ends after the ground truth retrieved by the previous test set (because there
                 # is a fold offset) so we need to add the missing ground truth values (values after
@@ -314,7 +327,7 @@ class TimeseriesModelScorer(object):
                 historical_df_after_max_ground_truth_date = historical_df[historical_df[self.time_variable] > max_ground_truth_date]
                 self._append_per_timeseries_ground_truth(
                     timeseries_identifier,
-                    historical_df_after_max_ground_truth_date[self.time_variable].astype(str),
+                    historical_df_after_max_ground_truth_date[self.time_variable].values,
                     historical_df_after_max_ground_truth_date[self.target_variable].values,
                 )
 
@@ -335,7 +348,7 @@ class TimeseriesModelScorer(object):
         )
 
     def _compute_per_timeseries_metrics(self, historical_df, test_df, forecasts_by_timeseries, fold_id, append_forecasts):
-        if append_forecasts and not self.forecasts:  # retrieve historical values
+        if append_forecasts:  # retrieve historical values
             self._set_forecasts_ground_truth(historical_df)
 
         per_timeseries_metrics = {}
@@ -352,6 +365,9 @@ class TimeseriesModelScorer(object):
                 append_forecasts=append_forecasts
             )
 
+            if append_forecasts:
+                self._clean_forecasts_ground_truths(timeseries_identifier)
+
         return per_timeseries_metrics
 
     def _parse_custom_metrics(self):
@@ -412,6 +428,30 @@ class TimeseriesModelScorer(object):
         
         return per_timeseries_metrics, timeseries_aggregated_metrics
 
+    def score_without_historical_df(self, df_with_forecast, compute_aggregated_metrics):
+        """
+        Calculate per-timeseries metrics without the need of historical_df to have a naive_error and eventually compute aggregated metrics
+        :param pd.DataFrame df_with_forecast: DataFrame with at least forecast and timeseries identifiers columns
+        :param bool compute_aggregated_metrics: if True, aggreagate per-timeseriesmetrics
+        :return: (per_timeseries_metrics, timeseries_aggregated_metrics)
+        """
+        self.custom_metrics = self._parse_custom_metrics()
+        per_timeseries_metrics = {}
+        for timeseries_identifier, df_of_timeseries_identifier in timeseries_iterator(df_with_forecast, self.timeseries_identifier_columns):
+            per_timeseries_metrics[timeseries_identifier] = self._compute_timeseries_metrics(
+                y_true=df_of_timeseries_identifier[self.target_variable].to_numpy(),
+                forecast_values=df_of_timeseries_identifier["forecast"].to_numpy(),
+                quantiles_forecasts=np.row_stack([df_of_timeseries_identifier[build_quantile_column_name(q)].to_numpy() for q in self.quantiles])
+            )
+
+        metrics_df = pd.DataFrame.from_dict(per_timeseries_metrics, orient="index")
+        per_timeseries_metrics = self.filter_per_timeseries_metrics(per_timeseries_metrics)
+        timeseries_aggregated_metrics = None
+        if compute_aggregated_metrics:
+            timeseries_aggregated_metrics = self.aggregate_metrics_per_timeseries(metrics_df)
+
+        return per_timeseries_metrics, timeseries_aggregated_metrics
+
     def remove_naninf(self, score):
         """
         Format properly the perf.json file:
@@ -484,8 +524,9 @@ class TimeseriesModelScorer(object):
             self.forecasts[timeseries_identifier][JSONForecast.FUTURE_TIME] = timeseries_forecast[ModelForecast.TIMESTAMPS]
             self.forecasts[timeseries_identifier][JSONForecast.FUTURE_FORECAST_VALUES] = timeseries_forecast[ModelForecast.FORECAST_VALUES]
             train_df_of_timeseries_identifier = get_dataframe_of_timeseries_identifier(train_df, timeseries_identifier)[-self.PAST_TIME_STEPS_TO_INCLUDE:]
-            train_df_of_timeseries_identifier = train_df_of_timeseries_identifier.loc[train_df_of_timeseries_identifier[self.time_variable] > self.forecasts[timeseries_identifier][JSONForecast.GROUND_TRUTH_TIME].max()]
-            self.forecasts[timeseries_identifier][JSONForecast.FUTURE_FORECAST_CONTEXT_GROUND_TRUTH_TIME] = train_df_of_timeseries_identifier[self.time_variable].astype(str).values
+            max_ground_truth_date = pd.to_datetime(self.forecasts[timeseries_identifier][JSONForecast.GROUND_TRUTH_TIME]).max()
+            train_df_of_timeseries_identifier = train_df_of_timeseries_identifier.loc[train_df_of_timeseries_identifier[self.time_variable] > max_ground_truth_date]
+            self.forecasts[timeseries_identifier][JSONForecast.FUTURE_FORECAST_CONTEXT_GROUND_TRUTH_TIME] = format_forecast_dates(train_df_of_timeseries_identifier[self.time_variable].values)
             self.forecasts[timeseries_identifier][JSONForecast.FUTURE_FORECAST_CONTEXT_VALUES] = train_df_of_timeseries_identifier[self.target_variable].values
 
             quantiles_forecasts = timeseries_forecast[ModelForecast.QUANTILES_FORECASTS]
@@ -517,7 +558,7 @@ class TimeseriesModelScorer(object):
             quantile["futureForecast"] = [dku_nonaninf(v) for v in quantile["futureForecast"]]
         return quantile
 
-    def _compute_timeseries_metrics(self, y_true, forecast_values, quantiles_forecasts, naive_error):
+    def _compute_timeseries_metrics(self, y_true, forecast_values, quantiles_forecasts, naive_error=None):
         """
         Compute the time series metrics, given the ground truth (y_true) and the forecast values (forecast_values)
 
@@ -531,6 +572,10 @@ class TimeseriesModelScorer(object):
         kept_quantiles_forecasts = quantiles_forecasts[:, indices_to_keep]
 
         median_forecasts = kept_quantiles_forecasts[self.quantiles.index(0.5)]  # 0.5 must always be a quantile
+        if np.all(median_forecasts==0.):
+            logger.warning("Median forecast is missing, using forecast instead")
+            median_forecasts = kept_forecast_values
+
         lower_quantile = kept_quantiles_forecasts[0]
         upper_quantile = kept_quantiles_forecasts[-1]
         alpha = 1 - (self.quantiles[-1] - self.quantiles[0])
@@ -540,13 +585,14 @@ class TimeseriesModelScorer(object):
             "mse": mean_squared_error(kept_y_true, kept_forecast_values),
             "abs_error": abs_error(kept_y_true, median_forecasts),
             "abs_target_sum": abs_target_sum(kept_y_true),
-            "mase": mean_absolute_scaled_error(kept_y_true, median_forecasts, naive_error),
             "mape": mean_absolute_percentage_error(kept_y_true, median_forecasts),
             "smape": symetric_mean_absolute_percentage_error(kept_y_true, median_forecasts),
             "mae": mean_absolute_error(kept_y_true, kept_forecast_values),
-            "msis": mean_scaled_interval_error(kept_y_true, lower_quantile, upper_quantile, alpha, naive_error),
             "test_size": len(kept_y_true),  # used to compute weighted average
         }
+        if not self.skip_scoring:
+            metrics["mase"] = mean_absolute_scaled_error(kept_y_true, median_forecasts, naive_error)
+            metrics["msis"] = mean_scaled_interval_error(kept_y_true, lower_quantile, upper_quantile, alpha, naive_error)
 
         if self.custom_metrics:
             self._compute_timeseries_custom_metrics(metrics, kept_y_true, kept_forecast_values)
@@ -579,20 +625,20 @@ class TimeseriesModelScorer(object):
 
     def _append_per_timeseries_ground_truth(self, timeseries_identifier, dates, y_true):
         for key, value_to_append in [
-            (JSONForecast.GROUND_TRUTH_TIME, dates),
+            (JSONForecast.GROUND_TRUTH_TIME, format_forecast_dates(dates)),
             (JSONForecast.GROUND_TRUTH_VALUES, y_true),
         ]:
             self.forecasts[timeseries_identifier][key] = np.concatenate(
                 [self.forecasts[timeseries_identifier][key], value_to_append]
             )
 
-    def _append_per_timeseries_forecasts(self, timeseries_identifier, dates, y_true, forecast_values, quantiles_forecasts, fold_id):
+    def _append_per_timeseries_forecasts(self, timeseries_identifier, forecast_dates, y_true, forecast_values, quantiles_forecasts, fold_id):
         for key, value_to_append in [
-            (JSONForecast.GROUND_TRUTH_TIME, dates),
+            (JSONForecast.GROUND_TRUTH_TIME, forecast_dates),
             (JSONForecast.GROUND_TRUTH_VALUES, y_true),
-            (JSONForecast.FORECAST_TIME, dates),
+            (JSONForecast.FORECAST_TIME, forecast_dates),
             (JSONForecast.FORECAST_VALUES, forecast_values),
-            (JSONForecast.FORECAST_FOLD_ID, [fold_id for _ in dates]),
+            (JSONForecast.FORECAST_FOLD_ID, [fold_id for _ in forecast_dates]),
         ]:
             self.forecasts[timeseries_identifier][key] = np.concatenate(
                 [self.forecasts[timeseries_identifier][key], value_to_append]
@@ -608,21 +654,39 @@ class TimeseriesModelScorer(object):
         for timeseries_identifier, historical_df_of_timeseries_identifier in timeseries_iterator(
             historical_df, self.timeseries_identifier_columns
         ):
+            if not self.forecasts or not timeseries_identifier in self.forecasts:
+                self.forecasts[timeseries_identifier] = {
+                    JSONForecast.GROUND_TRUTH_TIME: [],
+                    JSONForecast.GROUND_TRUTH_VALUES: [],
+                    JSONForecast.FORECAST_TIME: [],
+                    JSONForecast.FORECAST_VALUES: [],
+                    JSONForecast.FORECAST_FOLD_ID: [],
+                    JSONForecast.QUANTILES: [
+                        {
+                            JSONForecast.QUANTILE: quantile,
+                            JSONForecast.FORECAST_VALUES: [],
+                        }
+                        for quantile in self.quantiles
+                    ],
+                }
+
             historical_df_of_timeseries_identifier = historical_df_of_timeseries_identifier[-self.PAST_TIME_STEPS_TO_INCLUDE:]
-            self.forecasts[timeseries_identifier] = {
-                JSONForecast.GROUND_TRUTH_TIME: historical_df_of_timeseries_identifier[-self.PAST_TIME_STEPS_TO_INCLUDE:][self.time_variable].astype(str),
-                JSONForecast.GROUND_TRUTH_VALUES: historical_df_of_timeseries_identifier[self.target_variable],
-                JSONForecast.FORECAST_TIME: [],
-                JSONForecast.FORECAST_VALUES: [],
-                JSONForecast.FORECAST_FOLD_ID: [],
-                JSONForecast.QUANTILES: [
-                    {
-                        JSONForecast.QUANTILE: quantile,
-                        JSONForecast.FORECAST_VALUES: [],
-                    }
-                    for quantile in self.quantiles
-                ],
-            }
+            self._append_per_timeseries_ground_truth(timeseries_identifier, historical_df_of_timeseries_identifier[self.time_variable].values, historical_df_of_timeseries_identifier[self.target_variable].values)
+
+    def _clean_forecasts_ground_truths(self, timeseries_identifier):
+
+        # To avoid complex intersection and union logic between intervals, we concat all used ground truths during scoring
+        # then we remove duplicates and ensure sorting here
+        ts_ground_truth_df = pd.DataFrame({
+            JSONForecast.GROUND_TRUTH_TIME: pd.to_datetime(self.forecasts[timeseries_identifier][JSONForecast.GROUND_TRUTH_TIME]),
+            JSONForecast.GROUND_TRUTH_VALUES: self.forecasts[timeseries_identifier][JSONForecast.GROUND_TRUTH_VALUES],
+        })
+        ts_ground_truth_df = ts_ground_truth_df.drop_duplicates(subset=JSONForecast.GROUND_TRUTH_TIME, keep='first')
+        ts_ground_truth_df = ts_ground_truth_df.sort_values(JSONForecast.GROUND_TRUTH_TIME).reset_index(drop=True)
+
+        self.forecasts[timeseries_identifier][JSONForecast.GROUND_TRUTH_TIME] = format_forecast_dates(ts_ground_truth_df[JSONForecast.GROUND_TRUTH_TIME].values)
+        self.forecasts[timeseries_identifier][JSONForecast.GROUND_TRUTH_VALUES] = ts_ground_truth_df[JSONForecast.GROUND_TRUTH_VALUES].values
+
 
     def aggregate_metrics_per_timeseries(self, metrics_df):
         # TODO @timeseries: harmonize param names (snake vs camel cases)
@@ -634,7 +698,7 @@ class TimeseriesModelScorer(object):
         for quantile in self.quantiles:
             sum_aggregation_metrics += ["quantile_loss_{}".format(quantile)]
 
-        weighted_average_aggregation_metrics = SINGLE_TIMESERIES_METRICS.copy()
+        weighted_average_aggregation_metrics = SINGLE_TIMESERIES_METRICS.copy() if not self.skip_scoring else SINGLE_TIMESERIES_METRICS_SKIP_SCORING.copy()
 
         if self.custom_metrics:
             for custom_metric in self.custom_metrics:
@@ -646,9 +710,10 @@ class TimeseriesModelScorer(object):
             aggregated_metrics[metric_name] = np.sum(metrics_df[metric_name])
 
         for metric_name in weighted_average_aggregation_metrics:
-            metric_without_nans = metrics_df[metric_name].dropna()
-            weight_without_nans = metrics_df["test_size"][metric_without_nans.index]
-            aggregated_metrics[metric_name] = (metric_without_nans * weight_without_nans).sum() / weight_without_nans.sum()
+            if metric_name in metrics_df.columns:
+                metric_without_nans = metrics_df[metric_name].dropna()
+                weight_without_nans = metrics_df["test_size"][metric_without_nans.index]
+                aggregated_metrics[metric_name] = (metric_without_nans * weight_without_nans).sum() / weight_without_nans.sum()
 
         aggregated_metrics["rmse"] = np.sqrt(aggregated_metrics["mse"])
         aggregated_metrics["nd"] = aggregated_metrics["abs_error"] / aggregated_metrics["abs_target_sum"] if aggregated_metrics["abs_target_sum"] != 0 else np.nan
@@ -660,7 +725,8 @@ class TimeseriesModelScorer(object):
             [aggregated_metrics["quantile_loss_{}".format(quantile)] for quantile in self.quantiles]
         ) / aggregated_metrics["abs_target_sum"] if aggregated_metrics["abs_target_sum"] != 0 else np.nan 
 
-        metrics_to_return = AGGREGATED_TIMESERIES_METRICS
+        metrics_to_return = AGGREGATED_TIMESERIES_METRICS if not self.skip_scoring else AGGREGATED_TIMESERIES_METRICS_SKIP_SCORING
+
         if self.custom_metrics:
             metrics_to_return += [prefix_custom_metric_name(custom_metric["name"]) for custom_metric in self.custom_metrics]
 
@@ -672,7 +738,7 @@ class TimeseriesModelScorer(object):
         horizon, exactly like Trivial Identity), it is used in the MASE and MSIS metrics to compare errors to
         the naive error.
         """
-        all_target = np.append(historical_df[self.target_variable].to_numpy(), y_true)
+        all_target = np.append(historical_df[self.target_variable].values, y_true)
         season_shift_target = all_target[: -self.prediction_length]
         target = all_target[self.prediction_length :]
         naive_error = np.mean(abs(target - season_shift_target))
@@ -693,7 +759,7 @@ class TimeseriesModelScorer(object):
         return metrics_dict
 
     def filter_per_timeseries_metrics(self, per_timeseries_metrics):
-        metrics_to_return = SINGLE_TIMESERIES_METRICS
+        metrics_to_return = SINGLE_TIMESERIES_METRICS_SKIP_SCORING.copy() if self.skip_scoring else SINGLE_TIMESERIES_METRICS.copy()
         if self.custom_metrics:
             metrics_to_return += [prefix_custom_metric_name(custom_metric["name"]) for custom_metric in self.custom_metrics]
         return {
